Polynomials
Overview
This crate contains several algorithms over polynomials used in Starkom (NTT, Lagrange interpolation, etc.).
Most algorithms are implemented generically for any prime field. The main requirement is that the
field implements the
PrimeField trait provided in
the starkom-ff crate.
Structs
- Polynomial
- A polynomial expressed as an array of scalar coefficients in ascending degree order (i.e. the first coefficient is the constant term).